require 'studio_game/player'
require 'studio_game/treasure_trove'

module StudioGame
	describe Player do
		before do
			$stdout = StringIO.new
			# keeps studio_game.rb output from mingling with rspec output
			@initial_health = 150
			@player = Player.new("moe", @initial_health)
		end

		it "has a capitalized name" do 
			@player.name.should == "Moe"
			# expect(@player.name).to eq("Moe")
		end

		it "has a string representation" do 
			@player.found_treasure(Treasure.new(:hammer, 50))
			@player.to_s.should == "I'm Moe with a health = 150, points = 50, and score = 200"
		end
		it "computes a score as a sum of health and points" do
			@player.found_treasure(Treasure.new(:hammer, 50))
			@player.found_treasure(Treasure.new(:hammer, 50))

			@player.score.should == 250
		end
		it "increases health by 15 when w00ted" do
			@player.w00t
			@player.health.should == @initial_health + 15
			# expect(@player.health).to eq(@initial_health + 15)
		end
		it "decreases health by 10 when blammed" do
			@player.blam
			@player.health.should == @initial_health - 10
			# expect(@player.health).to eq(@initial_health - 10)
		end

		it "computes points as the sum of all treasure points" do
			@player.points.should == 0

			@player.found_treasure(Treasure.new(:hammer, 50))

			@player.points.should == 50

			@player.found_treasure(Treasure.new(:crowbar, 400))

			@player.points.should == 450

			@player.found_treasure(Treasure.new(:hammer, 50))

			@player.points.should == 500
		end

		it "yields each found treasure and its total points" do
			@player.found_treasure(Treasure.new(:skillet, 100))
			@player.found_treasure(Treasure.new(:skillet, 100))
			@player.found_treasure(Treasure.new(:hammer, 50))
			@player.found_treasure(Treasure.new(:bottle, 5))
			@player.found_treasure(Treasure.new(:bottle, 5))
			@player.found_treasure(Treasure.new(:bottle, 5))
			@player.found_treasure(Treasure.new(:bottle, 5))
			@player.found_treasure(Treasure.new(:bottle, 5))

			yielded = []
			@player.each_found_treasure do |treasure|
				yielded << treasure
			end

			yielded.should == [
				Treasure.new(:skillet, 200),
				Treasure.new(:hammer, 50),
				Treasure.new(:bottle, 25)
			]
		end

		context "created with a default health" do 
			before do
				@player = Player.new("moe")
			end
			it "has an initial health of 100" do
				@player.health.should == 100
				# expect(@player.health).to eq(100)
			end
		end

		context "player with a health of 150" do
			before do
				@initial_health = 150
				@player = Player.new("pat", @initial_health)
			end
			it "is strong" do
				@player.should be_strong
				# expect(@player).to be_strong
			end
		end

		context "player with a health of 100" do
			before do
				@initial_health = 100
				@player = Player.new("wuss", @initial_health)
			end

			it "is wimpy" do
				@player.should_not be_strong
				# expect(@player).not_to be_strong
			end
		end
		context "in a collection of players" do
		  	before do
				@player1 = Player.new("moe", 100)
				@player2 = Player.new("larry", 200)
				@player3 = Player.new("curly", 300)

				@players = [@player1, @player2, @player3]
		 	end

		  	it "is sorted by decreasing score" do
		   		@players.sort.should == [@player3, @player2, @player1]
		  	end
		end
	end
end